package com.gic.haoban.manage.api.service;

import com.gic.api.base.commons.Page;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.dto.role.StoreRoleDTO;

import java.util.List;

public interface StaffApiService {

    StaffDTO selectById(String staffId);

    List<StaffDTO> listByIds(List<String> staffIds);

    StaffDTO selectByNationcodeAndPhoneNumber(String wxEnterpriseId, String nationcode, String phoneNumber);

    List<StaffDepartmentRelatedDTO> listStaffDepartmentByStaffId(String staffId);

    Page<StaffDTO> pageStaff(List<String> departmentId, Integer activeFlag,
                             String keyword, BasePageInfo pageInfo, Integer relationFlag, String wxEnterpriseId);

    /**
     * 保存微信成员
     *
     * @param userId
     * @param wxEnterpriseId
     * @return
     */
    ServiceResponse getWxSaveNew(String userId, String wxEnterpriseId);

    /**
     * 新增成员
     *
     * @param userId
     * @param wxEnterpriseId
     */
    ServiceResponse wxGetAdd(String userId, String wxEnterpriseId);

    /**
     * 成员修改
     *
     * @param staffDTO
     * @param departmentIds
     */
    void staffEdit(StaffDTO staffDTO, String departmentIds);

    /**
     * 选择用户id和企业标识
     *
     * @param userId         用户id
     * @param wxEnterpriseId wx企业标识
     * @return {@link StaffDTO }
     * @author mozhu
     * @date 2022-01-13 15:02:22
     */
    StaffDTO selectByUserIdAndEnterpriseId(String userId, String wxEnterpriseId);

    /**
     * 没手机号的时候需要重新刷
     *
     * @param userId
     * @param wxEnterpriseId
     * @return
     */
    StaffDTO getByUserIdAndReflushWhenNoPhone(String userId, String wxEnterpriseId);

    StaffDepartmentRelatedDTO getDepartmentIdAndStaffId(String departmentId, String staffId);

    StaffDepartmentRelatedDTO getDepartmentIdAndCode(String departmentId, String code);

    void initWxUser(String corpid, String contactSecret, String wxEnterpriseId, Integer wxDepartmentId, String taskId);

    List<String> listUnBindClerk(String storeId);

    String insertUserLoginLog(UserLoginLogDTO userLoginLogDTO);

    int countByDepartmentIds(List<String> departmentIds);

    List<StaffDTO> listByPhoneNumber(String phoneNumber);

    /**
     * 获取成员头像
     *
     * @param clerkCode
     * @param storeId
     * @return
     */
    ServiceResponse<String> getStaffQrcode(String clerkCode, String storeId);

    void wxFristAdd(String userJson, String wxEnterpriseId);

    List<String> listBindStaffId(String storeId);

    List<StaffClerkRelationDTO> listBindRelationUserId(String storeId);

    /**
     * 激活
     *
     * @param staffId
     * @return
     */
    boolean activeStaff(String staffId);

    /**
     * 是否隐私
     *
     * @param staffId 员工id
     * @return {@link StaffPrivacyUseLogDTO }
     * @author mozhu
     * @date 2021-12-16 19:53:48
     */
    StaffPrivacyUseLogDTO getStaffPrivacyByStaffId(String staffId);


    /**
     * 让员工隐私员工id
     *
     * @param userId 用户id
     * @return {@link StaffPrivacyUseLogDTO }
     * @author mozhu
     * @date 2022-04-14 14:31:39
     */
    StaffPrivacyUseLogDTO getStaffPrivacyByUserId(String userId);


    /**
     * 用户id wx企业id列表
     *
     * @param wxEnterpriseId wx企业标识
     * @param pageNum        页面num
     * @param pageSize       页面大小
     * @return {@link List<StaffDTO> }
     * @author mozhu
     * @date 2022-02-25 09:40:15
     */
    Page<StaffDTO> listUserIdByWxEnterpriseId(String wxEnterpriseId, Integer pageNum, Integer pageSize);

    /**
     * desc: 查询当前品牌所有员工
     *
     * @param wxEnterpriseId
     * @return : {@link List< StaffDTO>}
     * @author : YongEn
     * @date : 2022/3/23
     */
    List<StaffDTO> listAllStaffByWxEnterpriseId(String wxEnterpriseId);


    /**
     * 通过开放用户id和wx企业id列表
     *
     * @param openUserIds    开放的用户id
     * @param wxEnterpriseId wx企业标识
     * @return {@link List<StaffDTO> }
     * @author mozhu
     * @date 2021-12-21 17:23:36
     */
    List<StaffDTO> listByOpenUserIdsAndWxEnterpriseId(List<String> openUserIds, String wxEnterpriseId);


    /**
     * 更新开放用户id,用户id
     *
     * @param wxEnterpriseId wx企业标识
     * @param wxUserId       wx用户id
     * @param wxOpenUseId    wx开放使用id
     * @return int
     * @author mozhu
     * @date 2021-12-21 19:53:30
     */
    int updateOpenUserIdsByUserId(String wxEnterpriseId, String wxUserId, String wxOpenUseId);


    /**
     * 更新开放id员工id
     *
     * @param staffId 员工id
     * @param openId  开放id
     * @return int
     */
    int updateOpenIdByStaffId(String staffId, String openId);

    /**
     * 更新二维码通过员工id
     *
     * @param staffId 员工id
     * @param qrCode  二维码
     * @return int
     */
    int updateQrCodeByStaffId(String staffId, String qrCode);


    /**
     * 获取职员id二维码
     *
     * @param clerkId 职员id
     * @return {@link StaffQrCodeDTO}
     */
    StaffQrCodeDTO getQrCodeByClerkId(String clerkId);

    /**
     * 导购gic门店和好办企业门店交集
     * 所有门店返回 -1
     *
     * @param clerkId        职员id
     * @param wxEnterpriseId wx企业标识
     * @return {@link List }<{@link String }>
     * @author mozhu
     * @date 2022-06-01 11:04:40
     */
    List<String> getHaoBanStoreIdsRolesByClerkId(String clerkId, String wxEnterpriseId);


    /**
     * 门店筛选器id和好办企业门店交集
     *
     * @param storeWidgetId  存储部件id
     * @param wxEnterpriseId wx企业标识
     * @param enterpriseId   企业标识
     * @return {@link List }<{@link String }>
     * @author mozhu
     * @date 2022-05-31 09:35:42
     */
    List<String> getHaoBanStoreRolesByStoreWidgetId(String storeWidgetId, String wxEnterpriseId, String enterpriseId);


    /**
     * 该门店好办是否有权限 true有，false 否
     *
     * @param storeId        存储id
     * @param enterpriseId   企业标识
     * @param wxEnterpriseId wx企业标识
     * @return boolean
     * @author mozhu
     * @date 2022-05-18 14:19:11
     */
    boolean isFlagByStoreIdAndEnterpriseId(String storeId, String enterpriseId, String wxEnterpriseId);


    /**
     * 是否超级管理员 true是
     *
     * @param clerkId 职员id
     * @return boolean
     * @author mozhu
     * @date 2022-05-27 09:43:00
     */
    boolean isSuperAdmin(String clerkId);


    /**
     * 获取到好办的企业门店id
     *
     * @param enterpriseId   企业标识
     * @param wxEnterpriseId wx企业标识
     * @return {@link List }<{@link String }>
     * @author mozhu
     * @date 2022-05-30 11:46:08
     */
    List<String> getHaoBanStoreRolesByEnterpriseId(String enterpriseId, String wxEnterpriseId);


    /**
     *
     * 获取区经管辖门店数量
     *
     * @param wxEnterpriseId wx企业标识
     * @param clerkId        职员id
     * @return int
     */
    StoreRoleDTO getCountHaoBanStoreIdsByClerkId(String clerkId, String wxEnterpriseId);

    /**
     * 是否店长权限 true 是
     * @param clerkId
     * @return
     */
    boolean isManager(String clerkId);

}
